@@ -185,11 +185,6 @@ static libvlc_media_list_player_t *getMediaListPlayer(JNIEnv *env, jobject thiz)
     return (libvlc_media_list_player_t*)(intptr_t)getLong(env, thiz, "mMediaListPlayerInstance");
 }
 
-static void unsetMediaPlayer(JNIEnv *env, jobject thiz)
-{
-    setLong(env, thiz, "mInternalMediaPlayerInstance", (jlong)0);
-}
-
 static void releaseMediaPlayer(JNIEnv *env, jobject thiz)
 {
     libvlc_media_list_player_t* p_mlp = getMediaListPlayer(env, thiz);
@@ -199,8 +194,8 @@ static void releaseMediaPlayer(JNIEnv *env, jobject thiz)
         libvlc_media_list_player_release(p_mlp);
         /* libvlc_media_list_player_release frees the media player, so
          * we don't free it ourselves. */
-        unsetMediaPlayer(env, thiz);
         setLong(env, thiz, "mInternalMediaPlayerInstance", 0);
+        setLong(env, thiz, "mMediaListPlayerInstance", 0);
     }
 }
 
@@ -254,8 +249,6 @@ static void vlc_event_callback(const libvlc_event_t *ev, void *data)
 
     int status = (*myVm)->GetEnv(myVm, (void**) &env, JNI_VERSION_1_2);
     if (status < 0) {
-        LOGD("vlc_event_callback: failed to get JNI environment, "
-             "assuming native thread");
         status = (*myVm)->AttachCurrentThread(myVm, &env, NULL);
         if (status < 0)
             return;
@@ -272,9 +265,14 @@ static void vlc_event_callback(const libvlc_event_t *ev, void *data)
     jobject bundle = (*env)->NewObject(env, clsBundle, clsCtor);
 
     jmethodID putInt = (*env)->GetMethodID(env, clsBundle, "putInt", "(Ljava/lang/String;I)V" );
+    jmethodID putFloat = (*env)->GetMethodID(env, clsBundle, "putFloat", "(Ljava/lang/String;F)V" );
     jmethodID putString = (*env)->GetMethodID(env, clsBundle, "putString", "(Ljava/lang/String;Ljava/lang/String;)V" );
 
-    if(ev->type == libvlc_MediaPlayerVout) {
+    if (ev->type == libvlc_MediaPlayerPositionChanged) {
+            jstring sData = (*env)->NewStringUTF(env, "data");
+            (*env)->CallVoidMethod(env, bundle, putFloat, sData, ev->u.media_player_position_changed.new_position);
+            (*env)->DeleteLocalRef(env, sData);
+    } else if(ev->type == libvlc_MediaPlayerVout) {
         /* For determining the vout/ES track change */
         jstring sData = (*env)->NewStringUTF(env, "data");
         (*env)->CallVoidMethod(env, bundle, putInt, sData, ev->u.media_player_vout.new_count);
@@ -412,6 +410,16 @@ void Java_org_videolan_vlc_LibVLC_nativeInit(JNIEnv *env, jobject thiz, jboolean
     methodId = (*env)->GetMethodID(env, cls, "timeStretchingEnabled", "()Z");
     bool enable_time_stretch = (*env)->CallBooleanMethod(env, thiz, methodId);
 
+    methodId = (*env)->GetMethodID(env, cls, "getChroma", "()Ljava/lang/String;");
+    jstring chroma = (*env)->CallObjectMethod(env, thiz, methodId);
+    const char *chromastr = (*env)->GetStringUTFChars(env, chroma, 0);
+    LOGD("Chroma set to \"%s\"", chromastr);
+
+    methodId = (*env)->GetMethodID(env, cls, "getSubtitlesEncoding", "()Ljava/lang/String;");
+    jstring subsencoding = (*env)->CallObjectMethod(env, thiz, methodId);
+    const char *subsencodingstr = (*env)->GetStringUTFChars(env, subsencoding, 0);
+    LOGD("Subtitle encoding set to \"%s\"", subsencodingstr);
+
     verbosity = verbose;
     libvlc_log_subscribe(&debug_subscriber, debug_log, &verbosity);
 
@@ -425,13 +433,18 @@ void Java_org_videolan_vlc_LibVLC_nativeInit(JNIEnv *env, jobject thiz, jboolean
         "--no-drop-late-frames",
         "--avcodec-fast",
         "--avcodec-threads=0",
+        "--subsdec-encoding", subsencodingstr,
         enable_time_stretch ? "--audio-time-stretch" : "--no-audio-time-stretch",
         use_opensles ? "--aout=opensles" : "--aout=android_audiotrack",
+        "--androidsurface-chroma", chromastr != NULL && chromastr[0] != 0 ? chromastr : "RV32",
     };
     libvlc_instance_t *instance = libvlc_new(sizeof(argv) / sizeof(*argv), argv);
 
     setLong(env, thiz, "mLibVlcInstance", (jlong)(intptr_t) instance);
 
+    (*env)->ReleaseStringUTFChars(env, chroma, chromastr);
+    (*env)->ReleaseStringUTFChars(env, subsencoding, subsencodingstr);
+
     if (!instance)
     {
         jclass exc = (*env)->FindClass(env, "org/videolan/vlc/LibVlcException");
@@ -596,6 +609,7 @@ static void create_player_and_play(JNIEnv* env, jobject thiz,
         libvlc_MediaPlayerEndReached,
         libvlc_MediaPlayerStopped,
         libvlc_MediaPlayerVout,
+        libvlc_MediaPlayerPositionChanged
     };
     for(int i = 0; i < (sizeof(mp_events) / sizeof(*mp_events)); i++)
         libvlc_event_attach(ev, mp_events[i], vlc_event_callback, myVm);
@@ -690,7 +704,6 @@ jboolean Java_org_videolan_vlc_LibVLC_hasVideoTrack(JNIEnv *env, jobject thiz,
     }
 
     /* Get the tracks information of the media. */
-    libvlc_media_track_info_t *p_tracks;
     libvlc_media_parse(p_m);
 
     libvlc_media_player_t* p_mp = libvlc_media_player_new_from_media(p_m);
@@ -763,9 +776,9 @@ jobjectArray read_track_info_internal(JNIEnv *env, jobject thiz, libvlc_media_t*
     }
 
     /* Get the tracks information of the media. */
-    libvlc_media_track_info_t *p_tracks;
+    libvlc_media_track_t **p_tracks;
 
-    int i_nbTracks = libvlc_media_get_tracks_info(p_m, &p_tracks);
+    int i_nbTracks = libvlc_media_tracks_get(p_m, &p_tracks);
     jobjectArray array = (*env)->NewObjectArray(env, i_nbTracks + 1, cls, NULL);
 
     unsigned i;
@@ -791,26 +804,26 @@ jobjectArray read_track_info_internal(JNIEnv *env, jobject thiz, libvlc_media_t*
                 continue;
             }
 
-            setInt(env, item, "Id", p_tracks[i].i_id);
-            setInt(env, item, "Type", p_tracks[i].i_type);
-            setString(env, item, "Codec", (const char*)vlc_fourcc_GetDescription(0,p_tracks[i].i_codec));
-            setString(env, item, "Language", p_tracks[i].psz_language);
+            setInt(env, item, "Id", p_tracks[i]->i_id);
+            setInt(env, item, "Type", p_tracks[i]->i_type);
+            setString(env, item, "Codec", (const char*)vlc_fourcc_GetDescription(0,p_tracks[i]->i_codec));
+            setString(env, item, "Language", p_tracks[i]->psz_language);
 
-            if (p_tracks[i].i_type == libvlc_track_video)
+            if (p_tracks[i]->i_type == libvlc_track_video)
             {
-                setInt(env, item, "Height", p_tracks[i].u.video.i_height);
-                setInt(env, item, "Width", p_tracks[i].u.video.i_width);
-                setFloat(env, item, "Framerate", p_tracks[i].u.video.f_frame_rate);
+                setInt(env, item, "Height", p_tracks[i]->video->i_height);
+                setInt(env, item, "Width", p_tracks[i]->video->i_width);
+                setFloat(env, item, "Framerate", (float)p_tracks[i]->video->i_frame_rate_num / p_tracks[i]->video->i_frame_rate_den);
             }
-            if (p_tracks[i].i_type == libvlc_track_audio)
+            if (p_tracks[i]->i_type == libvlc_track_audio)
             {
-                setInt(env, item, "Channels", p_tracks[i].u.audio.i_channels);
-                setInt(env, item, "Samplerate", p_tracks[i].u.audio.i_rate);
+                setInt(env, item, "Channels", p_tracks[i]->audio->i_channels);
+                setInt(env, item, "Samplerate", p_tracks[i]->audio->i_rate);
             }
         }
     }
 
-    libvlc_media_tracks_info_release(p_tracks, i_nbTracks);
+    libvlc_media_tracks_release(p_tracks, i_nbTracks);
     return array;
 }
 
@@ -1026,7 +1039,7 @@ jint Java_org_videolan_vlc_LibVLC_getAudioTracksCount(JNIEnv *env, jobject thiz)
     return -1;
 }
 
-jobjectArray Java_org_videolan_vlc_LibVLC_getAudioTrackDescription(JNIEnv *env, jobject thiz)
+jobject Java_org_videolan_vlc_LibVLC_getAudioTrackDescription(JNIEnv *env, jobject thiz)
 {
     libvlc_media_player_t *mp = getMediaPlayer(env, thiz);
     if (!mp)
@@ -1035,21 +1048,38 @@ jobjectArray Java_org_videolan_vlc_LibVLC_getAudioTrackDescription(JNIEnv *env,
     int i_nbTracks = libvlc_audio_get_track_count(mp) - 1;
     if (i_nbTracks < 0)
         i_nbTracks = 0;
-    jobjectArray array = (*env)->NewObjectArray(env, i_nbTracks,
-            (*env)->FindClass(env, "java/lang/String"),
-            NULL);
+    jclass mapClass = (*env)->FindClass(env, "java/util/Map");
+    jclass hashMapClass = (*env)->FindClass(env, "java/util/HashMap");
+    jmethodID mapPut = (*env)->GetMethodID(env, mapClass, "put", "(Ljava/lang/Object;Ljava/lang/Object;)Ljava/lang/Object;");
+    /*
+     * "What are you building? Lay your hand on it. Where is it?"
+     * We need a concrete map to start
+     */
+    jmethodID mapInit = (*env)->GetMethodID(env, hashMapClass, "<init>", "()V");
+    jclass integerCls = (*env)->FindClass(env, "java/lang/Integer");
+    jmethodID integerConstructor = (*env)->GetMethodID(env, integerCls, "<init>", "(I)V");
+
+    jobject audioTrackMap = (*env)->NewObject(env, hashMapClass, mapInit);
 
     libvlc_track_description_t *first = libvlc_audio_get_track_description(mp);
     libvlc_track_description_t *desc = first != NULL ? first->p_next : NULL;
     unsigned i;
     for (i = 0; i < i_nbTracks; ++i)
     {
+        // store audio track ID and name in a map as <ID, Track Name>
+        jobject track_id = (*env)->NewObject(env, integerCls, integerConstructor, desc->i_id);
         jstring name = (*env)->NewStringUTF(env, desc->psz_name);
-        (*env)->SetObjectArrayElement(env, array, i, name);
+        (*env)->CallObjectMethod(env, audioTrackMap, mapPut, track_id, name);
         desc = desc->p_next;
     }
     libvlc_track_description_list_release(first);
-    return array;
+
+    // Clean up local references
+    (*env)->DeleteLocalRef(env, mapClass);
+    (*env)->DeleteLocalRef(env, hashMapClass);
+    (*env)->DeleteLocalRef(env, integerCls);
+
+    return audioTrackMap;
 }
 
 jint Java_org_videolan_vlc_LibVLC_getAudioTrack(JNIEnv *env, jobject thiz)
