@@ -20,7 +20,15 @@
 
 package org.videolan.vlc;
 
+import java.io.BufferedReader;
+import java.io.BufferedWriter;
+import java.io.FileInputStream;
+import java.io.FileOutputStream;
+import java.io.IOException;
+import java.io.InputStreamReader;
+import java.io.OutputStreamWriter;
 import java.util.ArrayList;
+import java.util.Calendar;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Stack;
@@ -70,7 +78,9 @@ public class AudioService extends Service {
     public static final String ACTION_REMOTE_PAUSE = "org.videolan.vlc.remote.Pause";
     public static final String ACTION_REMOTE_STOP = "org.videolan.vlc.remote.Stop";
     public static final String ACTION_REMOTE_FORWARD = "org.videolan.vlc.remote.Forward";
+    public static final String ACTION_REMOTE_LAST_PLAYLIST = "org.videolan.vlc.remote.LastPlaylist";
     public static final String ACTION_WIDGET_UPDATE = "org.videolan.vlc.widget.UPDATE";
+    public static final String ACTION_WIDGET_UPDATE_POSITION = "org.videolan.vlc.widget.UPDATE_POSITION";
 
     public static final String WIDGET_PACKAGE = "org.videolan.vlc";
     public static final String WIDGET_CLASS = "org.videolan.vlc.widget.VLCAppWidgetProvider";
@@ -102,13 +112,17 @@ public class AudioService extends Service {
      */
     private boolean mLibVLCPlaylistActive = false;
 
+    /**
+     * Last widget position update timestamp
+     */
+    private long mWidgetPositionTimestamp = Calendar.getInstance().getTimeInMillis();
+
     @Override
     public void onCreate() {
         super.onCreate();
 
         // Get libVLC instance
         try {
-            LibVLC.useIOMX(this);
             mLibVLC = LibVLC.getInstance();
         } catch (LibVlcException e) {
             e.printStackTrace();
@@ -136,8 +150,10 @@ public class AudioService extends Service {
         filter.addAction(ACTION_REMOTE_PAUSE);
         filter.addAction(ACTION_REMOTE_STOP);
         filter.addAction(ACTION_REMOTE_FORWARD);
+        filter.addAction(ACTION_REMOTE_LAST_PLAYLIST);
         filter.addAction(Intent.ACTION_HEADSET_PLUG);
         filter.addAction(AudioManager.ACTION_AUDIO_BECOMING_NOISY);
+        filter.addAction(VLCApplication.SLEEP_INTENT);
         registerReceiver(serviceReceiver, filter);
 
         final SharedPreferences pref = PreferenceManager.getDefaultSharedPreferences(this);
@@ -303,6 +319,8 @@ public class AudioService extends Service {
                 stop();
             } else if (action.equalsIgnoreCase(ACTION_REMOTE_FORWARD)) {
                 next();
+            } else if (action.equalsIgnoreCase(ACTION_REMOTE_LAST_PLAYLIST)) {
+                loadLastPlaylist();
             }
 
             /*
@@ -320,6 +338,13 @@ public class AudioService extends Service {
                         play();
                 }
             }
+
+            /*
+             * Sleep
+             */
+            if (action.equalsIgnoreCase(VLCApplication.SLEEP_INTENT)) {
+                stop();
+            }
         }
     };
 
@@ -341,6 +366,28 @@ public class AudioService extends Service {
             switch (msg.getData().getInt("event")) {
                 case EventManager.MediaPlayerPlaying:
                     Log.i(TAG, "MediaPlayerPlaying");
+
+                    if (service.mCurrentMedia == null)
+                        return;
+                    String location = service.mCurrentMedia.getLocation();
+                    long length = service.mLibVLC.getLength();
+                    DatabaseManager dbManager = DatabaseManager
+                            .getInstance(VLCApplication.getAppContext());
+                    Media m = dbManager.getMedia(VLCApplication.getAppContext(),
+                            location);
+                    /**
+                     * 1) There is a media to update
+                     * 2) It has a length of 0
+                     * (dynamic track loading - most notably the OGG container)
+                     * 3) We were able to get a length even after parsing
+                     * (don't want to replace a 0 with a 0)
+                     */
+                    if(m != null && m.getLength() == 0 && length > 0) {
+                        Log.d(TAG, "Updating audio file length");
+                        dbManager.updateMedia(location,
+                                DatabaseManager.mediaColumn.MEDIA_LENGTH, length);
+                    }
+
                     service.changeAudioFocus(true);
                     service.setRemoteControlClientPlaybackState(RemoteControlClient.PLAYSTATE_PLAYING);
                     if (!service.mWakeLock.isHeld())
@@ -374,6 +421,10 @@ public class AudioService extends Service {
                         service.handleVout();
                     }
                     break;
+                case EventManager.MediaPlayerPositionChanged:
+                    float pos = msg.getData().getFloat("data");
+                    service.updateWidgetPosition(service, pos);
+                    break;
                 default:
                     Log.e(TAG, "Event not handled");
                     break;
@@ -441,11 +492,10 @@ public class AudioService extends Service {
             .setLargeIcon(AudioUtil.getCover(this, mCurrentMedia, 64))
             .setContentTitle(mCurrentMedia.getTitle())
             .setTicker(mCurrentMedia.getTitle() + " - " + mCurrentMedia.getArtist())
-            .setContentText((Util.isJellyBeanOrLater() ? mCurrentMedia.getArtist()
-                    : mCurrentMedia.getArtist() + " - " + mCurrentMedia.getAlbum()))
-                    .setContentInfo(mCurrentMedia.getAlbum())
-                    .setAutoCancel(false)
-                    .setOngoing(true);
+            .setContentText(Util.isJellyBeanOrLater() ? mCurrentMedia.getArtist() : mCurrentMedia.getSubtitle())
+            .setContentInfo(mCurrentMedia.getAlbum())
+            .setAutoCancel(false)
+            .setOngoing(true);
 
             Intent notificationIntent = new Intent(this, AudioPlayerActivity.class);
             notificationIntent.setAction(Intent.ACTION_MAIN);
@@ -485,8 +535,8 @@ public class AudioService extends Service {
     }
 
     private void stop() {
-        mEventManager.removeHandler(mEventHandler);
         mLibVLC.stop();
+        mEventManager.removeHandler(mEventHandler);
         setRemoteControlClientPlaybackState(RemoteControlClient.PLAYSTATE_STOPPED);
         mCurrentMedia = null;
         mMediaList.clear();
@@ -531,6 +581,7 @@ public class AudioService extends Service {
         showNotification();
         updateWidget(this);
         updateRemoteControlClientMetadata();
+        saveCurrentMedia();
     }
 
     @TargetApi(14)
@@ -573,12 +624,14 @@ public class AudioService extends Service {
         showNotification();
         updateWidget(this);
         updateRemoteControlClientMetadata();
+        saveCurrentMedia();
     }
 
     private void shuffle() {
         if (mShuffling)
             mPrevious.clear();
         mShuffling = !mShuffling;
+        saveCurrentMedia();
     }
 
     private void setRepeatType(int t) {
@@ -696,7 +749,7 @@ public class AudioService extends Service {
         }
 
         @Override
-        public void load(List<String> mediaPathList, int position, boolean libvlcBacked)
+        public void load(List<String> mediaPathList, int position, boolean libvlcBacked, boolean noVideo)
                 throws RemoteException {
             mLibVLCPlaylistActive = libvlcBacked;
 
@@ -730,13 +783,15 @@ public class AudioService extends Service {
                 if(mLibVLCPlaylistActive) {
                     mLibVLC.playIndex(position);
                 } else {
-                    mLibVLC.readMedia(mCurrentMedia.getLocation());
+                    mLibVLC.readMedia(mCurrentMedia.getLocation(), noVideo);
                 }
                 setUpRemoteControlClient();
                 showNotification();
                 updateWidget(AudioService.this);
                 updateRemoteControlClientMetadata();
             }
+            AudioService.this.saveMediaList();
+            AudioService.this.saveCurrentMedia();
         }
 
         @Override
@@ -763,7 +818,9 @@ public class AudioService extends Service {
                     VLCApplication.getAppContext().getString(R.string.unknown_album),
                     0,
                     0,
-                    "");
+                    "",
+                    -1,
+                    -1);
             mMediaList.add(mCurrentMedia);
 
             // Notify everyone
@@ -775,7 +832,7 @@ public class AudioService extends Service {
         @Override
         public void append(List<String> mediaPathList) throws RemoteException {
             if (mMediaList.size() == 0) {
-                load(mediaPathList, 0, false);
+                load(mediaPathList, 0, false, false);
                 return;
             }
 
@@ -792,6 +849,7 @@ public class AudioService extends Service {
                 }
                 mMediaList.add(media);
             }
+            AudioService.this.saveMediaList();
         }
 
         @Override
@@ -892,4 +950,106 @@ public class AudioService extends Service {
 
         sendBroadcast(i);
     }
+
+    private void updateWidgetPosition(Context context, float pos)
+    {
+        // no more than one widget update for each 1/50 of the song
+        long timestamp = Calendar.getInstance().getTimeInMillis();
+        if (mCurrentMedia == null ||
+            timestamp - mWidgetPositionTimestamp < mCurrentMedia.getLength() / 50)
+            return;
+
+        mWidgetPositionTimestamp = timestamp;
+        Intent i = new Intent();
+        i.setClassName(WIDGET_PACKAGE, WIDGET_CLASS);
+        i.setAction(ACTION_WIDGET_UPDATE_POSITION);
+        i.putExtra("position", pos);
+        sendBroadcast(i);
+    }
+
+    private synchronized void loadLastPlaylist() {
+        if (!Util.hasExternalStorage())
+            return;
+
+        String line;
+        FileInputStream input;
+        BufferedReader br;
+        int rowCount = 0;
+
+        int position = 0;
+        String currentMedia;
+        List<String> mediaPathList = new ArrayList<String>();
+
+        try {
+            // read CurrentMedia
+            input = new FileInputStream(AudioUtil.CACHE_DIR + "/" + "CurrentMedia.txt");
+            br = new BufferedReader(new InputStreamReader(input));
+            currentMedia = br.readLine();
+            mShuffling = "1".equals(br.readLine());
+            br.close();
+            input.close();
+
+            // read MediaList
+            input = new FileInputStream(AudioUtil.CACHE_DIR + "/" + "MediaList.txt");
+            br = new BufferedReader(new InputStreamReader(input));
+            while ((line = br.readLine()) != null) {
+                mediaPathList.add(line);
+                if (line.equals(currentMedia))
+                    position = rowCount;
+                rowCount++;
+            }
+            br.close();
+            input.close();
+
+            // load playlist
+            mInterface.load(mediaPathList, position, false, false);
+        } catch (IOException e) {
+            e.printStackTrace();
+        } catch (RemoteException e) {
+            e.printStackTrace();
+        }
+    }
+
+    private synchronized void saveCurrentMedia() {
+        if (!Util.hasExternalStorage())
+            return;
+
+        FileOutputStream output;
+        BufferedWriter bw;
+
+        try {
+            output = new FileOutputStream(AudioUtil.CACHE_DIR + "/" + "CurrentMedia.txt");
+            bw = new BufferedWriter(new OutputStreamWriter(output));
+            bw.write(mCurrentMedia != null ? mCurrentMedia.getLocation() : "");
+            bw.write('\n');
+            bw.write(mShuffling ? "1" : "0");
+            bw.write('\n');
+            bw.close();
+            output.close();
+        } catch (IOException e) {
+            e.printStackTrace();
+        }
+    }
+
+    private synchronized void saveMediaList() {
+        if (!Util.hasExternalStorage())
+            return;
+
+        FileOutputStream output;
+        BufferedWriter bw;
+
+        try {
+            output = new FileOutputStream(AudioUtil.CACHE_DIR + "/" + "MediaList.txt");
+            bw = new BufferedWriter(new OutputStreamWriter(output));
+            for (int i = 0; i < mMediaList.size(); i++) {
+                Media item = mMediaList.get(i);
+                bw.write(item.getLocation());
+                bw.write('\n');
+            }
+            bw.close();
+            output.close();
+        } catch (IOException e) {
+            e.printStackTrace();
+        }
+    }
 }
