@@ -21,6 +21,7 @@
 package org.videolan.vlc;
 
 import java.util.ArrayList;
+import java.util.Map;
 
 import org.videolan.vlc.gui.video.VideoPlayerActivity;
 
@@ -33,13 +34,11 @@ import android.view.Surface;
 
 public class LibVLC {
     private static final String TAG = "VLC/LibVLC";
-    private static final int AOUT_AUDIOTRACK = 1;
     private static final int AOUT_AUDIOTRACK_JAVA = 0;
+    private static final int AOUT_AUDIOTRACK = 1;
     private static final int AOUT_OPENSLES = 2;
 
     private static LibVLC sInstance;
-    private static boolean sUseIomx = false;
-    private static int sAout = AOUT_AUDIOTRACK_JAVA;
 
     /** libVLC instance C pointer */
     private long mLibVlcInstance = 0; // Read-only, reserved for JNI
@@ -56,7 +55,6 @@ public class LibVLC {
 
     /** Check in libVLC already initialized otherwise crash */
     private boolean mIsInitialized = false;
-
     public native void attachSurface(Surface surface, VideoPlayerActivity player, int width, int height);
 
     public native void detachSurface();
@@ -144,41 +142,8 @@ public class LibVLC {
      */
     public native void setSurface(Surface f);
 
-    /**
-     *
-     */
-    public boolean useIOMX() {
-        return sUseIomx;
-    }
-
-    public static synchronized void setIOMX(boolean enable) {
-        sUseIomx = enable;
-    }
-
-    public int getAout() {
-        return sAout;
-    }
-
-    public static synchronized void useIOMX(Context context) {
-        SharedPreferences pref = PreferenceManager.getDefaultSharedPreferences(context);
-        sUseIomx = pref.getBoolean("enable_iomx", false);
-        try {
-            sAout = Integer.parseInt(pref.getString("aout", String.valueOf(AOUT_AUDIOTRACK_JAVA)));
-        } catch (NumberFormatException nfe) {
-            sAout = AOUT_AUDIOTRACK_JAVA;
-        }
-    }
-
-    public static synchronized void setAout(Context context, Integer aoutPref,
-            boolean reset) {
-        if (aoutPref == AOUT_AUDIOTRACK)
-            sAout = AOUT_AUDIOTRACK;
-        else if (aoutPref == AOUT_OPENSLES && Util.isGingerbreadOrLater())
-            sAout = AOUT_OPENSLES;
-        else
-            sAout = AOUT_AUDIOTRACK_JAVA;
-
-        if (reset && sInstance != null) {
+    public static synchronized void restart() {
+        if (sInstance != null) {
             try {
                 sInstance.destroy();
                 sInstance.init();
@@ -188,10 +153,38 @@ public class LibVLC {
         }
     }
 
+    /**
+     * those are called from native code to get settings values
+     */
+    public boolean useIOMX() {
+        final SharedPreferences p = PreferenceManager.getDefaultSharedPreferences(VLCApplication.getAppContext());
+        return  p.getBoolean("enable_iomx", false);
+    }
+    public String getSubtitlesEncoding() {
+        final SharedPreferences p = PreferenceManager.getDefaultSharedPreferences(VLCApplication.getAppContext());
+        return p.getString("subtitles_text_encoding", "");
+    }
+    public int getAout() {
+        final SharedPreferences p = PreferenceManager.getDefaultSharedPreferences(VLCApplication.getAppContext());
+        int defaultAout = Util.isGingerbreadOrLater() ? AOUT_OPENSLES : AOUT_AUDIOTRACK_JAVA;
+        int aout = defaultAout;
+        try {
+            aout = Integer.parseInt(p.getString("aout", String.valueOf(defaultAout)));
+        }
+        catch (NumberFormatException nfe) {
+            aout = defaultAout;
+        }
+        return aout;
+    }
     public boolean timeStretchingEnabled() {
-        SharedPreferences p = PreferenceManager.getDefaultSharedPreferences(VLCApplication.getAppContext());
+        final SharedPreferences p = PreferenceManager.getDefaultSharedPreferences(VLCApplication.getAppContext());
         return p.getBoolean("enable_time_stretching_audio", false);
     }
+    public String getChroma() {
+        final SharedPreferences p = PreferenceManager.getDefaultSharedPreferences(VLCApplication.getAppContext());
+        String chroma = p.getString("chroma_format", "");
+        return chroma.equals("YV12") && !Util.isGingerbreadOrLater() ? "" : chroma;
+    }
 
     /**
      * Initialize the libVLC class
@@ -341,6 +334,7 @@ public class LibVLC {
      * Read a media
      * @param instance: the instance of libVLC
      * @param mrl: the media mrl
+     * @param novideo: don't enable video decoding for this media
      * @return the position in the playlist
      */
     private native int readMedia(long instance, String mrl, boolean novideo);
@@ -469,7 +463,7 @@ public class LibVLC {
 
     public native int getAudioTracksCount();
 
-    public native String[] getAudioTrackDescription();
+    public native Map<Integer,String> getAudioTrackDescription();
 
     public native int getAudioTrack();
 
@@ -485,7 +479,7 @@ public class LibVLC {
 
     public native int getSpuTracksCount();
 
-    public native String nativeToURI(String path);
+    public static native String nativeToURI(String path);
 
     public static native void nativeReadDirectory(String path, ArrayList<String> res);
 
