@@ -113,6 +113,8 @@ public class Media implements Comparable<Media> {
     private final String mLocation;
     private String mFilename;
     private long mTime = 0;
+    private int mAudioTrack = -1;
+    private int mSpuTrack = -1;
     private long mLength = 0;
     private int mType;
     private int mWidth = 0;
@@ -205,10 +207,12 @@ public class Media implements Comparable<Media> {
 
     public Media(Context context, String location, long time, long length, int type,
             Bitmap picture, String title, String artist, String genre, String album,
-            int width, int height, String artworkURL) {
+            int width, int height, String artworkURL, int audio, int spu) {
         mLocation = location;
         mFilename = null;
         mTime = time;
+        mAudioTrack = audio;
+        mSpuTrack = spu;
         mLength = length;
         mType = type;
         mPicture = picture;
@@ -254,6 +258,22 @@ public class Media implements Comparable<Media> {
         mTime = time;
     }
 
+    public int getAudioTrack() {
+        return mAudioTrack;
+    }
+
+    public void setAudioTrack(int track) {
+        mAudioTrack = track;
+    }
+
+    public int getSpuTrack() {
+        return mSpuTrack;
+    }
+
+    public void setSpuTrack(int track) {
+        mSpuTrack = track;
+    }
+
     public long getLength() {
         return mLength;
     }
@@ -314,7 +334,7 @@ public class Media implements Comparable<Media> {
     }
 
     public String getTitle() {
-        if (mTitle != null)
+        if (mTitle != null && mType != TYPE_VIDEO)
             return mTitle;
         else {
             int end = getFileName().lastIndexOf(".");
@@ -324,6 +344,10 @@ public class Media implements Comparable<Media> {
         }
     }
 
+    public String getSubtitle() {
+        return mType != TYPE_VIDEO ? mArtist + " - " + mAlbum : "";
+    }
+
     public String getArtist() {
         return mArtist;
     }
