@@ -30,16 +30,20 @@ import java.util.concurrent.locks.Lock;
 import java.util.concurrent.locks.ReentrantLock;
 
 import org.videolan.vlc.gui.MainActivity;
-import org.videolan.vlc.gui.video.VideoListFragment;
+import org.videolan.vlc.gui.video.VideoGridFragment;
 
+import android.content.Context;
 import android.graphics.Bitmap;
 import android.graphics.Bitmap.Config;
 import android.util.DisplayMetrics;
 import android.util.Log;
+import android.view.Display;
 
 public class ThumbnailerManager implements Runnable {
     public final static String TAG = "VLC/ThumbnailerManager";
 
+    private VideoGridFragment mVideoGridFragment;
+
     private final Queue<Media> mItems = new LinkedList<Media>();
 
     private boolean isStopping = false;
@@ -48,26 +52,34 @@ public class ThumbnailerManager implements Runnable {
 
     protected Thread mThread;
     private LibVLC mLibVlc;
-    private final VideoListFragment mVideoListActivity;
+    private final Context mContext;
     private int totalCount;
     private final float mDensity;
+    private final String mPrefix;
 
-    public ThumbnailerManager(VideoListFragment videoListActivity) {
-        mVideoListActivity = videoListActivity;
-        try {
-            mLibVlc = LibVLC.getInstance();
-        } catch (LibVlcException e) {
-            e.printStackTrace();
-        }
-
+    public ThumbnailerManager(Context context, Display display) {
+        mContext = context;
         DisplayMetrics metrics = new DisplayMetrics();
-        mVideoListActivity.getActivity().getWindowManager().getDefaultDisplay().getMetrics(metrics);
+        display.getMetrics(metrics);
         mDensity = metrics.density;
+        mPrefix = mContext.getResources().getString(R.string.thumbnail);
     }
 
-    public void start() {
+    public void start(VideoGridFragment videoGridFragment) {
+        if (mLibVlc == null) {
+            try {
+                mLibVlc = LibVLC.getInstance();
+            } catch (LibVlcException e) {
+                Log.e(TAG, "Can't obtain libvlc instance");
+                e.printStackTrace();
+                return;
+            }
+        }
+
+        isStopping = false;
         if (mThread == null || mThread.getState() == State.TERMINATED) {
-            isStopping = false;
+            mVideoGridFragment = videoGridFragment;
+            mVideoGridFragment.setThumbnailerManager(this);
             mThread = new Thread(this);
             mThread.start();
         }
@@ -75,6 +87,8 @@ public class ThumbnailerManager implements Runnable {
 
     public void stop() {
         isStopping = true;
+        if (mThread != null)
+            mThread.interrupt();
     }
 
     /**
@@ -110,24 +124,24 @@ public class ThumbnailerManager implements Runnable {
 
         Log.d(TAG, "Thumbnailer started");
 
-        String prefix = mVideoListActivity.getResources().getString(R.string.thumbnail);
-
         while (!isStopping) {
+            mVideoGridFragment.resetBarrier();
             lock.lock();
             // Get the id of the file browser item to create its thumbnail.
-            boolean killed = false;
+            boolean interrupted = false;
             while (mItems.size() == 0) {
                 try {
-                    MainActivity.hideProgressBar(mVideoListActivity.getActivity());
-                    MainActivity.clearTextInfo(mVideoListActivity.getActivity());
+                    MainActivity.hideProgressBar(mContext);
+                    MainActivity.clearTextInfo(mContext);
                     totalCount = 0;
                     notEmpty.await();
                 } catch (InterruptedException e) {
-                    killed = true;
+                    interrupted = true;
+                    Log.i(TAG, "interruption probably requested by stop()");
                     break;
                 }
             }
-            if (killed) {
+            if (interrupted) {
                 lock.unlock();
                 break;
             }
@@ -135,9 +149,9 @@ public class ThumbnailerManager implements Runnable {
             Media item = mItems.poll();
             lock.unlock();
 
-            MainActivity.showProgressBar(mVideoListActivity.getActivity());
+            MainActivity.showProgressBar(mContext);
 
-            MainActivity.sendTextInfo(mVideoListActivity.getActivity(), String.format("%s %s", prefix, item.getFileName()), count, total);
+            MainActivity.sendTextInfo(mContext, String.format("%s %s", mPrefix, item.getFileName()), count, total);
             count++;
 
             int width = (int) (120 * mDensity);
@@ -149,7 +163,7 @@ public class ThumbnailerManager implements Runnable {
             byte[] b = mLibVlc.getThumbnail(item.getLocation(), width, height);
 
             if (b == null) {// We were not able to create a thumbnail for this item.
-                item.setPicture(mVideoListActivity.getActivity(), null);
+                item.setPicture(mContext, null);
                 continue;
             }
 
@@ -158,19 +172,27 @@ public class ThumbnailerManager implements Runnable {
 
             Log.i(TAG, "Thumbnail created for " + item.getFileName());
 
-            item.setPicture(mVideoListActivity.getActivity(), thumbnail);
+            item.setPicture(mContext, thumbnail);
             // Post to the file browser the new item.
-            mVideoListActivity.setItemToUpdate(item);
+            mVideoGridFragment.setItemToUpdate(item);
 
             // Wait for the file browser to process the change.
             try {
-                mVideoListActivity.await();
+                mVideoGridFragment.await();
             } catch (InterruptedException e) {
+                Log.i(TAG, "interruption probably requested by stop()");
                 break;
             } catch (BrokenBarrierException e) {
+                Log.e(TAG, "Unexpected BrokenBarrierException");
+                e.printStackTrace();
                 break;
             }
         }
+        /* cleanup */
+        MainActivity.hideProgressBar(mContext);
+        MainActivity.clearTextInfo(mContext);
+        mVideoGridFragment.setThumbnailerManager(null);
+        mVideoGridFragment = null;
         Log.d(TAG, "Thumbnailer stopped");
     }
 }
