@@ -31,15 +31,18 @@ import java.io.InputStreamReader;
 import java.text.DecimalFormat;
 import java.text.NumberFormat;
 import java.util.ArrayList;
+import java.util.Arrays;
 import java.util.HashSet;
 import java.util.Locale;
 import java.util.Properties;
 import java.util.StringTokenizer;
 
 import android.content.Context;
+import android.content.SharedPreferences;
 import android.graphics.Bitmap;
 import android.net.Uri;
 import android.os.Environment;
+import android.preference.PreferenceManager;
 import android.telephony.TelephonyManager;
 import android.util.DisplayMetrics;
 import android.util.TypedValue;
@@ -80,13 +83,7 @@ public class Util {
     }
 
     public static String PathToURI(String path) {
-        String URI;
-        try {
-            URI = LibVLC.getInstance().nativeToURI(path);
-        } catch (LibVlcException e) {
-            URI = "";
-        }
-        return URI;
+        return LibVLC.nativeToURI(path);
     }
 
     public static String stripTrailingSlash(String s) {
@@ -407,8 +404,9 @@ public class Util {
             ArrayList<String> list = new ArrayList<String>();
             list.add(Environment.getExternalStorageDirectory().getPath());
             String line;
-            while ((line = bufReader.readLine()) != null) {
-                if (line.contains("vfat") || line.contains("/mnt")) {
+            while((line = bufReader.readLine()) != null) {
+                if(line.contains("vfat") || line.contains("exfat") ||
+                   line.contains("/mnt") || line.contains("/Removable")) {
                     StringTokenizer tokens = new StringTokenizer(line, " ");
                     String s = tokens.nextToken();
                     s = tokens.nextToken(); // Take the second token, i.e. mount point
@@ -417,12 +415,13 @@ public class Util {
                         continue;
 
                     if (line.contains("/dev/block/vold")) {
-                        if (!line.startsWith("/mnt/secure") &&
-                            !line.startsWith("/mnt/shell") &&
-                            !line.startsWith("/mnt/asec") &&
-                            !line.startsWith("/mnt/obb") &&
+                        if (!line.startsWith("tmpfs") &&
                             !line.startsWith("/dev/mapper") &&
-                            !line.startsWith("tmpfs")) {
+                            !s.startsWith("/mnt/secure") &&
+                            !s.startsWith("/mnt/shell") &&
+                            !s.startsWith("/mnt/asec") &&
+                            !s.startsWith("/mnt/obb")
+                            ) {
                             list.add(s);
                         }
                     }
@@ -446,4 +445,69 @@ public class Util {
         }
         return dirs;
     }
+
+    public static String[] getCustomDirectories() {
+        SharedPreferences preferences = PreferenceManager.getDefaultSharedPreferences(VLCApplication.getAppContext());
+        final String custom_paths = preferences.getString("custom_paths", "");
+        if(custom_paths.equals(""))
+            return new String[0];
+        else
+            return custom_paths.split(":");
+    }
+
+    public static String[] getMediaDirectories() {
+        ArrayList<String> list = new ArrayList<String>();
+        list.addAll(Arrays.asList(Util.getStorageDirectories()));
+        list.addAll(Arrays.asList(Util.getCustomDirectories()));
+        return list.toArray(new String[list.size()]);
+    }
+
+    public static void addCustomDirectory(String path) {
+        SharedPreferences preferences = PreferenceManager.getDefaultSharedPreferences(VLCApplication.getAppContext());
+
+        ArrayList<String> dirs = new ArrayList<String>(
+                Arrays.asList(getCustomDirectories()));
+        dirs.add(path);
+        StringBuilder builder = new StringBuilder();
+        builder.append(dirs.remove(0));
+        for(String s : dirs) {
+            builder.append(":");
+            builder.append(s);
+        }
+        SharedPreferences.Editor editor = preferences.edit();
+        editor.putString("custom_paths", builder.toString());
+        editor.commit();
+    }
+
+    public static void removeCustomDirectory(String path) {
+        SharedPreferences preferences = PreferenceManager.getDefaultSharedPreferences(VLCApplication.getAppContext());
+        if(!preferences.getString("custom_paths", "").contains(path))
+            return;
+        ArrayList<String> dirs = new ArrayList<String>(
+                Arrays.asList(preferences.getString("custom_paths", "").split(
+                        ":")));
+        dirs.remove(path);
+        String custom_path;
+        if(dirs.size() > 0) {
+            StringBuilder builder = new StringBuilder();
+            builder.append(dirs.remove(0));
+            for(String s : dirs) {
+                builder.append(":");
+                builder.append(s);
+            }
+            custom_path = builder.toString();
+        } else { // don't do unneeded extra work
+            custom_path = "";
+        }
+        SharedPreferences.Editor editor = preferences.edit();
+        editor.putString("custom_paths", custom_path);
+        editor.commit();
+    }
+
+    /**
+     * Get the formatted current playback speed in the form of 1.00x
+     */
+    public static String formatRateString(float rate) {
+        return String.format(java.util.Locale.US, "%.2fx", rate);
+    }
 }
