@@ -20,8 +20,6 @@
 
 package org.videolan.vlc.gui;
 
-import java.util.ArrayList;
-
 import org.videolan.vlc.AudioService;
 import org.videolan.vlc.AudioServiceController;
 import org.videolan.vlc.LibVLC;
@@ -140,9 +138,6 @@ public class MainActivity extends SherlockFragmentActivity {
             editor.commit();
         }
 
-        /* Start VLC prefs */
-        LibVLC.useIOMX(this);
-
         try {
             // Start LibVLC
             LibVLC.getInstance();
@@ -469,6 +464,11 @@ public class MainActivity extends SherlockFragmentActivity {
                 else
                     MediaLibrary.getInstance(this).loadMediaItems(this, true);
                 break;
+            // Restore last playlist
+            case R.id.ml_menu_last_playlist:
+                Intent i = new Intent(AudioService.ACTION_REMOTE_LAST_PLAYLIST);
+                sendBroadcast(i);
+                break;
             // Open MRL
             case R.id.ml_menu_open_mrl:
                 onOpenMRL();
@@ -611,9 +611,7 @@ public class MainActivity extends SherlockFragmentActivity {
                        * (for example, RTSP and TS streaming) where ES can be
                        * dynamically adapted rather than a simple scan.
                        */
-                      ArrayList<String> media = new ArrayList<String>();
-                      media.add(s);
-                      c.append(media);
+                      c.append(s);
                     }
                 };
                 task.execute();
