@@ -33,6 +33,7 @@ import android.content.Context;
 import android.content.DialogInterface;
 import android.content.Intent;
 import android.content.SharedPreferences;
+import android.content.SharedPreferences.OnSharedPreferenceChangeListener;
 import android.os.Bundle;
 import android.preference.CheckBoxPreference;
 import android.preference.EditTextPreference;
@@ -46,12 +47,13 @@ import android.util.Log;
 import android.widget.Toast;
 
 @SuppressWarnings("deprecation")
-public class PreferencesActivity extends PreferenceActivity {
+public class PreferencesActivity extends PreferenceActivity implements OnSharedPreferenceChangeListener {
 
     public final static String TAG = "VLC/PreferencesActivity";
 
     public final static String NAME = "VlcSharedPreferences";
     public final static String LAST_MEDIA = "LastMedia";
+    public final static String VIDEO_RESUME_TIME = "VideoResumeTime";
     public final static int RESULT_RESCAN = RESULT_FIRST_USER + 1;
 
     @Override
@@ -98,18 +100,6 @@ public class PreferencesActivity extends PreferenceActivity {
                     }
                 });
 
-        // HW decoding
-        CheckBoxPreference checkboxHW = (CheckBoxPreference) findPreference("enable_iomx");
-        checkboxHW.setOnPreferenceClickListener(
-                new OnPreferenceClickListener() {
-                    @Override
-                    public boolean onPreferenceClick(Preference preference) {
-                        CheckBoxPreference checkboxHW = (CheckBoxPreference) preference;
-                        LibVLC.setIOMX(checkboxHW.isChecked());
-                        return true;
-                    }
-                });
-
         // Headset detection option
         CheckBoxPreference checkboxHS = (CheckBoxPreference) findPreference("enable_headset_detection");
         checkboxHS.setOnPreferenceClickListener(
@@ -171,13 +161,10 @@ public class PreferencesActivity extends PreferenceActivity {
         int aoutEntriesIdValues = Util.isGingerbreadOrLater() ? R.array.aouts_values : R.array.aouts_values_froyo;
         aoutPref.setEntries(aoutEntriesId);
         aoutPref.setEntryValues(aoutEntriesIdValues);
-        aoutPref.setOnPreferenceChangeListener(new OnPreferenceChangeListener() {
-            @Override
-            public boolean onPreferenceChange(Preference preference, Object newValue) {
-                LibVLC.setAout(PreferencesActivity.this, Integer.valueOf((String) newValue), true);
-                return true;
-            }
-        });
+        if (aoutPref.getValue() == null)
+            aoutPref.setValue(Util.isGingerbreadOrLater()
+                ? "2"/*AOUT_OPENSLES*/
+                : "0"/*AOUT_AUDIOTRACK_JAVA*/);
 
         // Attach debugging items
         Preference quitAppPref = findPreference("quit_app");
@@ -210,6 +197,21 @@ public class PreferencesActivity extends PreferenceActivity {
                 return true;
             }
         });
+
+        // SharedPreferences Listener to apply changes
+        SharedPreferences sharedPrefs = PreferenceManager.getDefaultSharedPreferences(this);
+        sharedPrefs.registerOnSharedPreferenceChangeListener(this);
+    }
+
+    @Override
+    public void onSharedPreferenceChanged(SharedPreferences sharedPreferences, String key) {
+        if (key.equalsIgnoreCase("enable_iomx") ||
+            key.equalsIgnoreCase("subtitles_text_encoding") ||
+            key.equalsIgnoreCase("aout") ||
+            key.equalsIgnoreCase("enable_time_stretching_audio") ||
+            key.equalsIgnoreCase("chroma_format")) {
+                LibVLC.restart();
+        }
     }
 
     @Override
