@@ -1,7 +1,7 @@
 /*****************************************************************************
  * VideoPlayerActivity.java
  *****************************************************************************
- * Copyright © 2011-2012 VLC authors and VideoLAN
+ * Copyright © 2011-2013 VLC authors and VideoLAN
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,6 +24,9 @@ import java.io.File;
 import java.io.UnsupportedEncodingException;
 import java.lang.reflect.Method;
 import java.net.URLDecoder;
+import java.util.ArrayList;
+import java.util.Arrays;
+import java.util.Map;
 
 import org.videolan.vlc.AudioServiceController;
 import org.videolan.vlc.DatabaseManager;
@@ -33,9 +36,10 @@ import org.videolan.vlc.LibVlcException;
 import org.videolan.vlc.Media;
 import org.videolan.vlc.R;
 import org.videolan.vlc.Util;
+import org.videolan.vlc.VLCApplication;
 import org.videolan.vlc.WeakHandler;
+import org.videolan.vlc.gui.AdvFuncDialog;
 import org.videolan.vlc.gui.PreferencesActivity;
-import org.videolan.vlc.gui.SpeedSelectorDialog;
 import org.videolan.vlc.gui.audio.AudioPlayerActivity;
 import org.videolan.vlc.interfaces.IPlayerControl;
 import org.videolan.vlc.interfaces.OnPlayerControlListener;
@@ -54,7 +58,9 @@ import android.content.IntentFilter;
 import android.content.SharedPreferences;
 import android.content.pm.ActivityInfo;
 import android.content.res.Configuration;
+import android.database.Cursor;
 import android.graphics.Color;
+import android.graphics.ImageFormat;
 import android.graphics.PixelFormat;
 import android.media.AudioManager;
 import android.os.Build;
@@ -62,6 +68,7 @@ import android.os.Bundle;
 import android.os.Handler;
 import android.os.Message;
 import android.preference.PreferenceManager;
+import android.provider.MediaStore;
 import android.provider.Settings.SettingNotFoundException;
 import android.text.format.DateFormat;
 import android.util.DisplayMetrics;
@@ -78,10 +85,8 @@ import android.view.View.OnSystemUiVisibilityChangeListener;
 import android.view.ViewGroup.LayoutParams;
 import android.view.WindowManager;
 import android.view.animation.AnimationUtils;
-import android.widget.ArrayAdapter;
 import android.widget.FrameLayout;
 import android.widget.ImageButton;
-import android.widget.ListView;
 import android.widget.SeekBar;
 import android.widget.SeekBar.OnSeekBarChangeListener;
 import android.widget.TextView;
@@ -92,6 +97,8 @@ public class VideoPlayerActivity extends Activity {
 
     private SurfaceView mSurface;
     private SurfaceHolder mSurfaceHolder;
+    private FrameLayout mSurfaceFrame;
+    private int mSurfaceAlign;
     private LibVLC mLibVLC;
     private String mLocation;
 
@@ -134,8 +141,9 @@ public class VideoPlayerActivity extends Activity {
     private ImageButton mSubtitle;
     private ImageButton mLock;
     private ImageButton mSize;
-    private TextView mSpeedLabel;
     private boolean mIsLocked = false;
+    private int mLastAudioTrack = -1;
+    private int mLastSpuTrack = -1;
 
     /**
      * For uninterrupted switching between audio and video mode
@@ -165,10 +173,12 @@ public class VideoPlayerActivity extends Activity {
     private boolean mIsFirstBrightnessGesture = true;
 
     // Tracks & Subtitles
-    private String[] mAudioTracksLibVLC;
-    private ArrayAdapter<String> mAudioTracksAdapter;
+    private Map<Integer,String> mAudioTracksList;
     private String[] mSubtitleTracksLibVLC;
-    private ArrayAdapter<String> mSubtitleTracksAdapter;
+    private ArrayList<String> mSubtitleTracksList;
+
+    // Advance Function
+    private AdvFuncDialog mAdvFuncDialog;
 
     @Override
     @TargetApi(11)
@@ -237,22 +247,35 @@ public class VideoPlayerActivity extends Activity {
                  * The setTracksAndSubtitles method probably doesn't work in case of many many Tracks and Subtitles
                  * Moreover, in a video stream, if Tracks & Subtitles change, they won't be updated
                  */
-                setTracksAndSubtitles();
+                setESTrackLists();
             }}, 1500);
 
-
         mLock = (ImageButton) findViewById(R.id.lock_overlay_button);
         mLock.setOnClickListener(mLockListener);
 
         mSize = (ImageButton) findViewById(R.id.player_overlay_size);
         mSize.setOnClickListener(mSizeListener);
 
-        mSpeedLabel = (TextView) findViewById(R.id.player_overlay_speed);
-        mSpeedLabel.setOnClickListener(mSpeedLabelListener);
-
         mSurface = (SurfaceView) findViewById(R.id.player_surface);
         mSurfaceHolder = mSurface.getHolder();
-        mSurfaceHolder.setFormat(PixelFormat.RGBX_8888);
+        mSurfaceFrame = (FrameLayout) findViewById(R.id.player_surface_frame);
+        int pitch;
+        String chroma = pref.getString("chroma_format", "");
+        if(Util.isGingerbreadOrLater() && chroma.equals("YV12")) {
+            mSurfaceHolder.setFormat(ImageFormat.YV12);
+            pitch = ImageFormat.getBitsPerPixel(ImageFormat.YV12) / 8;
+        } else if (chroma.equals("RV16")) {
+            mSurfaceHolder.setFormat(PixelFormat.RGB_565);
+            PixelFormat info = new PixelFormat();
+            PixelFormat.getPixelFormatInfo(PixelFormat.RGB_565, info);
+            pitch = info.bytesPerPixel;
+        } else {
+            mSurfaceHolder.setFormat(PixelFormat.RGBX_8888);
+            PixelFormat info = new PixelFormat();
+            PixelFormat.getPixelFormatInfo(PixelFormat.RGBX_8888, info);
+            pitch = info.bytesPerPixel;
+        }
+        mSurfaceAlign = 16 / pitch - 1;
         mSurfaceHolder.addCallback(mSurfaceCallback);
 
         mSeekbar = (SeekBar) findViewById(R.id.player_overlay_seekbar);
@@ -264,10 +287,19 @@ public class VideoPlayerActivity extends Activity {
         mSwitchingView = false;
         mEndReached = false;
 
-        registerReceiver(mBatteryReceiver, new IntentFilter(Intent.ACTION_BATTERY_CHANGED));
+        // Clear the resume time, since it is only used for resumes in external
+        // videos.
+        SharedPreferences preferences = getSharedPreferences(PreferencesActivity.NAME, MODE_PRIVATE);
+        SharedPreferences.Editor editor = preferences.edit();
+        editor.putLong(PreferencesActivity.VIDEO_RESUME_TIME, -1);
+        editor.commit();
+
+        IntentFilter filter = new IntentFilter();
+        filter.addAction(Intent.ACTION_BATTERY_CHANGED);
+        filter.addAction(VLCApplication.SLEEP_INTENT);
+        registerReceiver(mReceiver, filter);
 
         try {
-            LibVLC.useIOMX(this);
             mLibVLC = LibVLC.getInstance();
         } catch (LibVlcException e) {
             Log.d(TAG, "LibVLC initialisation failed");
@@ -335,21 +367,33 @@ public class VideoPlayerActivity extends Activity {
         if (time >= 0) {
             SharedPreferences preferences = getSharedPreferences(PreferencesActivity.NAME, MODE_PRIVATE);
             SharedPreferences.Editor editor = preferences.edit();
-            editor.putString(PreferencesActivity.LAST_MEDIA, mLocation);
+            if(DatabaseManager.getInstance(this).mediaItemExists(mLocation)) {
+                editor.putString(PreferencesActivity.LAST_MEDIA, mLocation);
+                DatabaseManager.getInstance(this).updateMedia(
+                        mLocation,
+                        DatabaseManager.mediaColumn.MEDIA_TIME,
+                        time);
+            } else {
+                // Video file not in media library, store time just for onResume()
+                editor.putLong(PreferencesActivity.VIDEO_RESUME_TIME, time);
+            }
             editor.commit();
-            DatabaseManager.getInstance(this).updateMedia(
-                    mLocation,
-                    DatabaseManager.mediaColumn.MEDIA_TIME,
-                    time);
         }
 
         AudioServiceController.getInstance().unbindAudioService(this);
     }
 
+    @Override
+    protected void onStop() {
+        super.onStop();
+        if (mAdvFuncDialog != null)
+            mAdvFuncDialog.destroyAdvFuncDialog();
+    }
+
     @Override
     protected void onDestroy() {
         super.onDestroy();
-        unregisterReceiver(mBatteryReceiver);
+        unregisterReceiver(mReceiver);
         if (mLibVLC != null && !mSwitchingView) {
             mLibVLC.stop();
         }
@@ -416,19 +460,25 @@ public class VideoPlayerActivity extends Activity {
         context.startActivity(intent);
     }
 
-    private final BroadcastReceiver mBatteryReceiver = new BroadcastReceiver()
+    private final BroadcastReceiver mReceiver = new BroadcastReceiver()
     {
         @Override
         public void onReceive(Context context, Intent intent)
         {
-            int batteryLevel = intent.getIntExtra("level", 0);
-            if (batteryLevel >= 50)
-                mBattery.setTextColor(Color.GREEN);
-            else if  (batteryLevel >= 30)
-                mBattery.setTextColor(Color.YELLOW);
-            else
-                mBattery.setTextColor(Color.RED);
-            mBattery.setText(String.format("%d%%", batteryLevel));
+            String action = intent.getAction();
+            if (action.equalsIgnoreCase(Intent.ACTION_BATTERY_CHANGED)) {
+                int batteryLevel = intent.getIntExtra("level", 0);
+                if (batteryLevel >= 50)
+                    mBattery.setTextColor(Color.GREEN);
+                else if (batteryLevel >= 30)
+                    mBattery.setTextColor(Color.YELLOW);
+                else
+                    mBattery.setTextColor(Color.RED);
+                mBattery.setText(String.format("%d%%", batteryLevel));
+            }
+            else if (action.equalsIgnoreCase(VLCApplication.SLEEP_INTENT)) {
+                finish();
+            }
         }
     };
 
@@ -445,6 +495,9 @@ public class VideoPlayerActivity extends Activity {
     }
 
     public void setSurfaceSize(int width, int height, int sar_num, int sar_den) {
+        if (width * height == 0)
+            return;
+
         // store video size
         mVideoHeight = height;
         mVideoWidth = width;
@@ -546,6 +599,7 @@ public class VideoPlayerActivity extends Activity {
             switch (msg.getData().getInt("event")) {
                 case EventManager.MediaPlayerPlaying:
                     Log.i(TAG, "MediaPlayerPlaying");
+                    activity.setESTracks();
                     break;
                 case EventManager.MediaPlayerPaused:
                     Log.i(TAG, "MediaPlayerPaused");
@@ -560,6 +614,9 @@ public class VideoPlayerActivity extends Activity {
                 case EventManager.MediaPlayerVout:
                     activity.handleVout(msg);
                     break;
+                case EventManager.MediaPlayerPositionChanged:
+                    //don't spam the logs
+                    break;
                 default:
                     Log.e(TAG, String.format("Event not handled (0x%x)", msg.getData().getInt("event")));
                     break;
@@ -690,11 +747,24 @@ public class VideoPlayerActivity extends Activity {
                 break;
         }
 
-        mSurfaceHolder.setFixedSize(mVideoWidth, mVideoHeight);
+        // align width on 16bytes
+        int alignedWidth = (mVideoWidth + mSurfaceAlign) & ~mSurfaceAlign;
+
+        // force surface buffer size
+        mSurfaceHolder.setFixedSize(alignedWidth, mVideoHeight);
+
+        // set display size
         LayoutParams lp = mSurface.getLayoutParams();
-        lp.width = dw;
+        lp.width = dw * alignedWidth / mVideoWidth;
         lp.height = dh;
         mSurface.setLayoutParams(lp);
+
+        // set frame size (crop if necessary)
+        lp = mSurfaceFrame.getLayoutParams();
+        lp.width = dw;
+        lp.height = dh;
+        mSurfaceFrame.setLayoutParams(lp);
+
         mSurface.invalidate();
     }
 
@@ -875,29 +945,43 @@ public class VideoPlayerActivity extends Activity {
     private final OnClickListener mAudioTrackListener = new OnClickListener() {
         @Override
         public void onClick(View v) {
+            final String[] arrList = new String[mAudioTracksList.size()];
+            int i = 0;
+            int listPosition = 0;
+            for(Map.Entry<Integer,String> entry : mAudioTracksList.entrySet()) {
+                arrList[i] = entry.getValue();
+                // map the track position to the list position
+                if(entry.getKey() == mLibVLC.getAudioTrack())
+                    listPosition = i;
+                i++;
+            }
             AlertDialog dialog = new AlertDialog.Builder(VideoPlayerActivity.this)
             .setTitle(R.string.track_audio)
-            .setAdapter(mAudioTracksAdapter, new DialogInterface.OnClickListener() {
+            .setSingleChoiceItems(arrList, listPosition, new DialogInterface.OnClickListener() {
                 @Override
-                public void onClick(DialogInterface dialog, int position) {
-                    mLibVLC.setAudioTrack(position + 1);
+                public void onClick(DialogInterface dialog, int listPosition) {
+                    int trackID = -1;
+                    // Reverse map search...
+                    for(Map.Entry<Integer, String> entry : mAudioTracksList.entrySet()) {
+                        if(arrList[listPosition].equals(entry.getValue())) {
+                            trackID = entry.getKey();
+                            break;
+                        }
+                    }
+                    if(trackID < 0) return;
+
+                    DatabaseManager.getInstance(VideoPlayerActivity.this).updateMedia(
+                            mLocation,
+                            DatabaseManager.mediaColumn.MEDIA_AUDIOTRACK,
+                            trackID);
+                    mLibVLC.setAudioTrack(trackID);
                     dialog.dismiss();
                 }
             })
             .create();
             dialog.setCanceledOnTouchOutside(true);
-            final ListView lv = dialog.getListView();
-            lv.setChoiceMode(ListView.CHOICE_MODE_SINGLE);
-            //FIXME We should not force background color
-            lv.setBackgroundColor(0xffffffff);
+            dialog.setOwnerActivity(VideoPlayerActivity.this);
             dialog.show();
-            lv.post(new Runnable() {
-
-                @Override
-                public void run() {
-                    lv.setItemChecked((mLibVLC.getAudioTrack() - 1), true);
-                }
-            });
         }
     };
 
@@ -907,29 +991,25 @@ public class VideoPlayerActivity extends Activity {
     private final OnClickListener mSubtitlesListener = new OnClickListener() {
         @Override
         public void onClick(View v) {
+            String[] arrList = new String[mSubtitleTracksList.size()];
+            arrList = mSubtitleTracksList.toArray(arrList);
             AlertDialog dialog = new AlertDialog.Builder(VideoPlayerActivity.this)
             .setTitle(R.string.track_text)
-            .setAdapter(mSubtitleTracksAdapter, new DialogInterface.OnClickListener() {
+            .setSingleChoiceItems(arrList, mLibVLC.getSpuTrack(), new DialogInterface.OnClickListener() {
                 @Override
                 public void onClick(DialogInterface dialog, int position) {
+                    DatabaseManager.getInstance(VideoPlayerActivity.this).updateMedia(
+                            mLocation,
+                            DatabaseManager.mediaColumn.MEDIA_SPUTRACK,
+                            position);
                     mLibVLC.setSpuTrack(position);
                     dialog.dismiss();
                 }
             })
             .create();
             dialog.setCanceledOnTouchOutside(true);
-            final ListView lv = dialog.getListView();
-            lv.setChoiceMode(ListView.CHOICE_MODE_SINGLE);
-            //FIXME We should not force background color
-            lv.setBackgroundColor(0xffffffff);
+            dialog.setOwnerActivity(VideoPlayerActivity.this);
             dialog.show();
-            lv.post(new Runnable() {
-
-                @Override
-                public void run() {
-                    lv.setItemChecked((mLibVLC.getSpuTrack()), true);
-                }
-            });
         }
     };
 
@@ -948,6 +1028,9 @@ public class VideoPlayerActivity extends Activity {
 
         @Override
         public void onSeek(int delta) {
+            // unseekable stream
+            if(mLibVLC.getLength() <= 0) return;
+
             long position = mLibVLC.getTime() + delta;
             if (position < 0) position = 0;
             mLibVLC.setTime(position);
@@ -956,6 +1039,8 @@ public class VideoPlayerActivity extends Activity {
 
         @Override
         public void onSeekTo(long position) {
+            // unseekable stream
+            if(mLibVLC.getLength() <= 0) return;
             mLibVLC.setTime(position);
             mTime.setText(Util.millisToString(position));
         }
@@ -1034,21 +1119,6 @@ public class VideoPlayerActivity extends Activity {
         }
     };
 
-    private final OnClickListener mSpeedLabelListener = new OnClickListener() {
-        @Override
-        public void onClick(View v) {
-            SpeedSelectorDialog d = new SpeedSelectorDialog(VideoPlayerActivity.this);
-            d.setOnDismissListener(new DialogInterface.OnDismissListener() {
-
-                @Override
-                public void onDismiss(DialogInterface dialog) {
-                    mSpeedLabel.setText(String.format(java.util.Locale.US, "%.2fx", mLibVLC.getRate()));
-                }
-            });
-            d.show();
-        }
-    };
-
     private final OnClickListener mRemainingTimeListener = new OnClickListener() {
         @Override
         public void onClick(View v) {
@@ -1063,6 +1133,14 @@ public class VideoPlayerActivity extends Activity {
     private final SurfaceHolder.Callback mSurfaceCallback = new Callback() {
         @Override
         public void surfaceChanged(SurfaceHolder holder, int format, int width, int height) {
+            if(format == PixelFormat.RGBX_8888)
+                Log.d(TAG, "Pixel format is RGBX_8888");
+            else if(format == PixelFormat.RGB_565)
+                Log.d(TAG, "Pixel format is RGB_565");
+            else if(format == ImageFormat.YV12)
+                Log.d(TAG, "Pixel format is YV12");
+            else
+                Log.d(TAG, "Pixel format is other/unknown");
             mLibVLC.attachSurface(holder.getSurface(), VideoPlayerActivity.this, width, height);
         }
 
@@ -1135,14 +1213,12 @@ public class VideoPlayerActivity extends Activity {
      */
     @TargetApi(11)
     private void dimStatusBar(boolean dim) {
-        if (!Util.isHoneycombOrLater())
+        if (!Util.isHoneycombOrLater() || !Util.hasNavBar())
             return;
         mSurface.setSystemUiVisibility(
                 dim ? (Util.hasCombBar()
                         ? View.SYSTEM_UI_FLAG_LOW_PROFILE
-                        : Util.hasNavBar()
-                            ? View.SYSTEM_UI_FLAG_HIDE_NAVIGATION
-                            : View.SYSTEM_UI_FLAG_LOW_PROFILE)
+                        : View.SYSTEM_UI_FLAG_HIDE_NAVIGATION)
                     : View.SYSTEM_UI_FLAG_VISIBLE);
     }
 
@@ -1165,6 +1241,7 @@ public class VideoPlayerActivity extends Activity {
         int length = (int) mLibVLC.getLength();
         // Update all view elements
 
+        mControls.setSeekable(length > 0);
         mSeekbar.setMax(length);
         mSeekbar.setProgress(time);
         mSysTime.setText(DateFormat.format("kk:mm", System.currentTimeMillis()));
@@ -1172,14 +1249,25 @@ public class VideoPlayerActivity extends Activity {
         mLength.setText(mDisplayRemainingTime && length > 0
                 ? "- " + Util.millisToString(length - time)
                 : Util.millisToString(length));
+
         return time;
     }
 
-    private void setTracksAndSubtitles () {
-        if (mAudioTracksLibVLC == null) {
-            mAudioTracksLibVLC = mLibVLC.getAudioTrackDescription();
-            if (mAudioTracksLibVLC != null && mAudioTracksLibVLC.length > 1) {
-                mAudioTracksAdapter = new ArrayAdapter<String>(this, R.layout.list_item_checkable, mAudioTracksLibVLC);
+    private void setESTracks() {
+        if (mLastAudioTrack >= 0) {
+            mLibVLC.setAudioTrack(mLastAudioTrack);
+            mLastAudioTrack = -1;
+        }
+        if (mLastSpuTrack >= 0) {
+            mLibVLC.setSpuTrack(mLastSpuTrack);
+            mLastSpuTrack = -1;
+        }
+    }
+
+    private void setESTrackLists() {
+        if(mAudioTracksList == null) {
+            if (mLibVLC.getAudioTracksCount() > 2) {
+                mAudioTracksList = mLibVLC.getAudioTrackDescription();
                 mAudioTrack.setOnClickListener(mAudioTrackListener);
                 mAudioTrack.setVisibility(View.VISIBLE);
             }
@@ -1191,7 +1279,7 @@ public class VideoPlayerActivity extends Activity {
         if (mSubtitleTracksLibVLC == null) {
             mSubtitleTracksLibVLC = mLibVLC.getSpuTrackDescription();
             if (mSubtitleTracksLibVLC != null && mSubtitleTracksLibVLC.length > 0) {
-                mSubtitleTracksAdapter = new ArrayAdapter<String>(this, R.layout.list_item_checkable, mSubtitleTracksLibVLC);
+                mSubtitleTracksList = new ArrayList<String>(Arrays.asList(mSubtitleTracksLibVLC));
                 mSubtitle.setOnClickListener(mSubtitlesListener);
                 mSubtitle.setVisibility(View.VISIBLE);
 
@@ -1223,6 +1311,7 @@ public class VideoPlayerActivity extends Activity {
     /**
      *
      */
+    @SuppressWarnings("deprecation")
     private void load() {
         mLocation = null;
         String title = getResources().getString(R.string.title);
@@ -1233,7 +1322,21 @@ public class VideoPlayerActivity extends Activity {
         if (getIntent().getAction() != null
                 && getIntent().getAction().equals(Intent.ACTION_VIEW)) {
             /* Started from external application */
-            mLocation = getIntent().getDataString();
+            if(getIntent().getData().getScheme().equals("content")) {
+                if(getIntent().getData().getHost().equals("media")) {
+                    // Media URI
+                    Cursor cursor = managedQuery(getIntent().getData(), new String[]{ MediaStore.Video.Media.DATA }, null, null, null);
+                    int column_index = cursor.getColumnIndexOrThrow(MediaStore.Video.Media.DATA);
+                    cursor.moveToFirst();
+                    mLocation = Util.PathToURI(cursor.getString(column_index));
+                } else {
+                    // other content-based URI (probably file pickers)
+                    mLocation = getIntent().getData().getPath();
+                }
+            } else {
+                // Plain URI
+                mLocation = getIntent().getDataString();
+            }
         } else if(getIntent().getExtras() != null) {
             /* Started from VideoListActivity */
             mLocation = getIntent().getExtras().getString("itemLocation");
@@ -1254,8 +1357,23 @@ public class VideoPlayerActivity extends Activity {
         if (mLocation != null && mLocation.length() > 0 && !dontParse) {
             // restore last position
             Media media = DatabaseManager.getInstance(this).getMedia(this, mLocation);
-            if (media != null && media.getTime() > 0 && !fromStart)
-                mLibVLC.setTime(media.getTime());
+            if(media != null) {
+                // in media library
+                if(media.getTime() > 0 && !fromStart)
+                    mLibVLC.setTime(media.getTime());
+
+                mLastAudioTrack = media.getAudioTrack();
+                mLastSpuTrack = media.getSpuTrack();
+            } else {
+                // not in media library
+                SharedPreferences preferences = getSharedPreferences(PreferencesActivity.NAME, MODE_PRIVATE);
+                long rTime = preferences.getLong(PreferencesActivity.VIDEO_RESUME_TIME, -1);
+                SharedPreferences.Editor editor = preferences.edit();
+                editor.putLong(PreferencesActivity.VIDEO_RESUME_TIME, -1);
+                editor.commit();
+                if(rTime > 0)
+                    mLibVLC.setTime(rTime);
+            }
 
             try {
                 title = URLDecoder.decode(mLocation, "UTF-8");
@@ -1309,4 +1427,10 @@ public class VideoPlayerActivity extends Activity {
             return 0;
         }
     }
+
+    public void showAdvanceFunction(View v) {
+        if (mAdvFuncDialog == null)
+            mAdvFuncDialog = new AdvFuncDialog(this);
+        mAdvFuncDialog.show();
+    }
 }
